@@ -16,13 +16,21 @@
  *
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include <babeltrace/ctf/types.h>
 #include <babeltrace/bitfield.h>
 #include <stdint.h>
 #include <glib.h>
-#include <endian.h>
+#include <babeltrace/endian.h>
 
 /*
  * The aligned read/write functions are expected to be faster than the
@@ -31,8 +39,8 @@
  */
 
 static
-int _aligned_integer_read(struct stream_pos *ppos,
-			  struct definition *definition)
+int _aligned_integer_read(struct bt_stream_pos *ppos,
+			  struct bt_definition *definition)
 {
 	struct definition_integer *integer_definition =
 		container_of(definition, struct definition_integer, p);
@@ -41,7 +49,8 @@ int _aligned_integer_read(struct stream_pos *ppos,
 	struct ctf_stream_pos *pos = ctf_pos(ppos);
 	int rbo = (integer_declaration->byte_order != BYTE_ORDER);	/* reverse byte order */
 
-	ctf_align_pos(pos, integer_declaration->p.alignment);
+	if (!ctf_align_pos(pos, integer_declaration->p.alignment))
+		return -EFAULT;
 
 	if (!ctf_pos_access_ok(pos, integer_declaration->len))
 		return -EFAULT;
@@ -53,7 +62,7 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			uint8_t v;
 
-			v = *(const uint8_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._unsigned = v;
 			break;
 		}
@@ -61,7 +70,7 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			uint16_t v;
 
-			v = *(const uint16_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._unsigned =
 				rbo ? GUINT16_SWAP_LE_BE(v) : v;
 			break;
@@ -70,7 +79,7 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			uint32_t v;
 
-			v = *(const uint32_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._unsigned =
 				rbo ? GUINT32_SWAP_LE_BE(v) : v;
 			break;
@@ -79,7 +88,7 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			uint64_t v;
 
-			v = *(const uint64_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._unsigned =
 				rbo ? GUINT64_SWAP_LE_BE(v) : v;
 			break;
@@ -93,7 +102,7 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			int8_t v;
 
-			v = *(const int8_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._signed = v;
 			break;
 		}
@@ -101,40 +110,41 @@ int _aligned_integer_read(struct stream_pos *ppos,
 		{
 			int16_t v;
 
-			v = *(const int16_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._signed =
-				rbo ? GUINT16_SWAP_LE_BE(v) : v;
+				rbo ? (int16_t) GUINT16_SWAP_LE_BE(v) : v;
 			break;
 		}
 		case 32:
 		{
 			int32_t v;
 
-			v = *(const int32_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._signed =
-				rbo ? GUINT32_SWAP_LE_BE(v) : v;
+				rbo ? (int32_t) GUINT32_SWAP_LE_BE(v) : v;
 			break;
 		}
 		case 64:
 		{
 			int64_t v;
 
-			v = *(const int64_t *) ctf_get_pos_addr(pos);
+			memcpy(&v, ctf_get_pos_addr(pos), sizeof(v));
 			integer_definition->value._signed =
-				rbo ? GUINT64_SWAP_LE_BE(v) : v;
+				rbo ? (int64_t) GUINT64_SWAP_LE_BE(v) : v;
 			break;
 		}
 		default:
 			assert(0);
 		}
 	}
-	ctf_move_pos(pos, integer_declaration->len);
+	if (!ctf_move_pos(pos, integer_declaration->len))
+		return -EFAULT;
 	return 0;
 }
 
 static
-int _aligned_integer_write(struct stream_pos *ppos,
-			    struct definition *definition)
+int _aligned_integer_write(struct bt_stream_pos *ppos,
+			    struct bt_definition *definition)
 {
 	struct definition_integer *integer_definition =
 		container_of(definition, struct definition_integer, p);
@@ -143,7 +153,8 @@ int _aligned_integer_write(struct stream_pos *ppos,
 	struct ctf_stream_pos *pos = ctf_pos(ppos);
 	int rbo = (integer_declaration->byte_order != BYTE_ORDER);	/* reverse byte order */
 
-	ctf_align_pos(pos, integer_declaration->p.alignment);
+	if (!ctf_align_pos(pos, integer_declaration->p.alignment))
+		return -EFAULT;
 
 	if (!ctf_pos_access_ok(pos, integer_declaration->len))
 		return -EFAULT;
@@ -152,58 +163,91 @@ int _aligned_integer_write(struct stream_pos *ppos,
 	if (pos->dummy)
 		goto end;
 	if (!integer_declaration->signedness) {
-		uint64_t v = integer_definition->value._unsigned;
-
 		switch (integer_declaration->len) {
-		case 8:	*(uint8_t *) ctf_get_pos_addr(pos) = (uint8_t) v;
+		case 8:
+		{
+			uint8_t v = integer_definition->value._unsigned;
+
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 16:
-			*(uint16_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT16_SWAP_LE_BE((uint16_t) v) :
-						 (uint16_t) v;
+		{
+			uint16_t v = integer_definition->value._unsigned;
+
+			if (rbo)
+				v = GUINT16_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 32:
-			*(uint32_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT32_SWAP_LE_BE((uint32_t) v) :
-						 (uint32_t) v;
+		{
+			uint32_t v = integer_definition->value._unsigned;
+
+			if (rbo)
+				v = GUINT32_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 64:
-			*(uint64_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT64_SWAP_LE_BE(v) : v;
+		{
+			uint64_t v = integer_definition->value._unsigned;
+
+			if (rbo)
+				v = GUINT64_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		default:
 			assert(0);
 		}
 	} else {
-		int64_t v = integer_definition->value._signed;
-
 		switch (integer_declaration->len) {
-		case 8:	*(int8_t *) ctf_get_pos_addr(pos) = (int8_t) v;
+		case 8:
+		{
+			uint8_t v = integer_definition->value._signed;
+
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 16:
-			*(int16_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT16_SWAP_LE_BE((int16_t) v) :
-						 (int16_t) v;
+		{
+			int16_t v = integer_definition->value._signed;
+
+			if (rbo)
+				v = GUINT16_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 32:
-			*(int32_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT32_SWAP_LE_BE((int32_t) v) :
-						 (int32_t) v;
+		{
+			int32_t v = integer_definition->value._signed;
+
+			if (rbo)
+				v = GUINT32_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		case 64:
-			*(int64_t *) ctf_get_pos_addr(pos) = rbo ?
-						 GUINT64_SWAP_LE_BE(v) : v;
+		{
+			int64_t v = integer_definition->value._signed;
+
+			if (rbo)
+				v = GUINT64_SWAP_LE_BE(v);
+			memcpy(ctf_get_pos_addr(pos), &v, sizeof(v));
 			break;
+		}
 		default:
 			assert(0);
 		}
 	}
 end:
-	ctf_move_pos(pos, integer_declaration->len);
+	if (!ctf_move_pos(pos, integer_declaration->len))
+		return -EFAULT;
 	return 0;
 }
 
-int ctf_integer_read(struct stream_pos *ppos, struct definition *definition)
+int ctf_integer_read(struct bt_stream_pos *ppos, struct bt_definition *definition)
 {
 	struct definition_integer *integer_definition =
 		container_of(definition, struct definition_integer, p);
@@ -216,35 +260,41 @@ int ctf_integer_read(struct stream_pos *ppos, struct definition *definition)
 		return _aligned_integer_read(ppos, definition);
 	}
 
-	ctf_align_pos(pos, integer_declaration->p.alignment);
+	if (!ctf_align_pos(pos, integer_declaration->p.alignment))
+		return -EFAULT;
 
 	if (!ctf_pos_access_ok(pos, integer_declaration->len))
 		return -EFAULT;
 
 	if (!integer_declaration->signedness) {
 		if (integer_declaration->byte_order == LITTLE_ENDIAN)
-			bt_bitfield_read_le(pos->base, unsigned long,
+			bt_bitfield_read_le(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				&integer_definition->value._unsigned);
 		else
-			bt_bitfield_read_be(pos->base, unsigned long,
+			bt_bitfield_read_be(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				&integer_definition->value._unsigned);
 	} else {
 		if (integer_declaration->byte_order == LITTLE_ENDIAN)
-			bt_bitfield_read_le(pos->base, unsigned long,
+			bt_bitfield_read_le(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				&integer_definition->value._signed);
 		else
-			bt_bitfield_read_be(pos->base, unsigned long,
+			bt_bitfield_read_be(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				&integer_definition->value._signed);
 	}
-	ctf_move_pos(pos, integer_declaration->len);
+	if (!ctf_move_pos(pos, integer_declaration->len))
+		return -EFAULT;
 	return 0;
 }
 
-int ctf_integer_write(struct stream_pos *ppos, struct definition *definition)
+int ctf_integer_write(struct bt_stream_pos *ppos, struct bt_definition *definition)
 {
 	struct definition_integer *integer_definition =
 		container_of(definition, struct definition_integer, p);
@@ -257,7 +307,8 @@ int ctf_integer_write(struct stream_pos *ppos, struct definition *definition)
 		return _aligned_integer_write(ppos, definition);
 	}
 
-	ctf_align_pos(pos, integer_declaration->p.alignment);
+	if (!ctf_align_pos(pos, integer_declaration->p.alignment))
+		return -EFAULT;
 
 	if (!ctf_pos_access_ok(pos, integer_declaration->len))
 		return -EFAULT;
@@ -266,24 +317,29 @@ int ctf_integer_write(struct stream_pos *ppos, struct definition *definition)
 		goto end;
 	if (!integer_declaration->signedness) {
 		if (integer_declaration->byte_order == LITTLE_ENDIAN)
-			bt_bitfield_write_le(pos->base, unsigned long,
+			bt_bitfield_write_le(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				integer_definition->value._unsigned);
 		else
-			bt_bitfield_write_be(pos->base, unsigned long,
+			bt_bitfield_write_be(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				integer_definition->value._unsigned);
 	} else {
 		if (integer_declaration->byte_order == LITTLE_ENDIAN)
-			bt_bitfield_write_le(pos->base, unsigned long,
+			bt_bitfield_write_le(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				integer_definition->value._signed);
 		else
-			bt_bitfield_write_be(pos->base, unsigned long,
+			bt_bitfield_write_be(mmap_align_addr(pos->base_mma) +
+					pos->mmap_base_offset, unsigned char,
 				pos->offset, integer_declaration->len,
 				integer_definition->value._signed);
 	}
 end:
-	ctf_move_pos(pos, integer_declaration->len);
+	if (!ctf_move_pos(pos, integer_declaration->len))
+		return -EFAULT;
 	return 0;
 }
