@@ -28,7 +28,7 @@
  */
 
 #ifdef __FreeBSD__
-#include <machine/endian.h>
+#include <sys/endian.h>
 
 #elif defined(__sun__)
 #include <sys/byteorder.h>
